@@ -74,6 +74,7 @@
 #include "PostgresBackend.h"
 #include "price.h"
 #include "txn.h"
+#include "txnmass.h"
 
 #include "putil.h"
 
@@ -290,25 +291,57 @@ static const char *table_drop_str =
 static gpointer
 query_cb (PGBackend *be, PGresult *result, int j, gpointer data)
 {
-   GList *node, *xaction_list = (GList *) data;
-   GUID *trans_guid;
+   GList *xaction_list = (GList *) data;
+   GUID trans_guid;
+   Transaction *trans;
+   gnc_commodity *currency;
+   Timespec ts;
 
    /* find the transaction this goes into */
-   trans_guid = xaccGUIDMalloc();
-   *trans_guid = nullguid;  /* just in case the read fails ... */
-   string_to_guid (DB_GET_VAL("transGUID",j), trans_guid);
+   trans_guid = nullguid;  /* just in case the read fails ... */
+   string_to_guid (DB_GET_VAL("transGUID",j), &trans_guid);
 
-   /* don't put transaction into the list more than once ... */
-   for (node=xaction_list; node; node=node->next)
+   /* use markers to avoid redundant traversals of transactions we've
+    * already checked recently. */
+   trans = xaccTransLookup (&trans_guid);
+   if (NULL != trans)
    {
-      if (guid_equal ((GUID *)node->data, trans_guid)) 
+      if (0 != trans->marker)
       {
-         xaccGUIDFree (trans_guid);
          return xaction_list;
       }
+      else
+      {
+         gint32 db_version, cache_version;
+         db_version = atoi (DB_GET_VAL("version",j));
+         cache_version = xaccTransGetVersion (trans);
+         if (db_version <= cache_version) {
+            return xaction_list;
+         }
+         xaccTransBeginEdit (trans);
+      }
+   }
+   else 
+   {
+      trans = xaccMallocTransaction();
+      xaccTransBeginEdit (trans);
+      xaccTransSetGUID (trans, &trans_guid);
    }
 
-   xaction_list = g_list_prepend (xaction_list, trans_guid);
+   xaccTransSetNum (trans, DB_GET_VAL("num",j));
+   xaccTransSetDescription (trans, DB_GET_VAL("description",j));
+   ts = gnc_iso8601_to_timespec_local (DB_GET_VAL("date_posted",j));
+   xaccTransSetDatePostedTS (trans, &ts);
+   ts = gnc_iso8601_to_timespec_local (DB_GET_VAL("date_entered",j));
+   xaccTransSetDateEnteredTS (trans, &ts);
+   xaccTransSetVersion (trans, atoi(DB_GET_VAL("version",j)));
+
+   currency = gnc_string_to_commodity (DB_GET_VAL("currency",j));
+   xaccTransSetCurrency (trans, currency);
+
+   trans->marker = 1;
+   xaction_list = g_list_prepend (xaction_list, trans);
+
    return xaction_list;
 }
 
@@ -322,88 +355,100 @@ static int ncalls = 0;
 static void 
 pgendFillOutToCheckpoint (PGBackend *be, const char *query_string)
 {
-   GList *node, *anode, *xaction_list= NULL, *acct_list = NULL;
+   int call_count = ncalls;
+   int nact=0;
+   GList *xaction_list = NULL;
+   GList *node, *anode, *acct_list = NULL;
 
    ENTER (" ");
    if (!be) return;
+
    ncalls ++;
 
    SEND_QUERY (be, query_string, );
-   xaction_list = pgendGetResults (be, query_cb, xaction_list);
-   if (NULL == xaction_list) return;
+   xaction_list = pgendGetResults (be, query_cb, NULL);
 
-   /* restore the transactions */
+   /* restore the splits for these transactions */
    for (node=xaction_list; node; node=node->next)
    {
-      Transaction *trans;
-      int engine_data_is_newer;
-      GUID *trans_guid = (GUID *)node->data;
+      Transaction *trans = (Transaction *) node->data;
+      GList *engine_splits, *snode;
+      gnc_commodity *currency = NULL;
+      currency = (gnc_commodity *) trans->common_currency;
 
-      /* use markers to avoid redundant traversals of transactions we've
-       * already checked recently. */
-      trans = xaccTransLookup (trans_guid);
-      if (NULL == trans || 0 == trans->marker)
+      pgendCopySplitsToEngine (be, trans);
+      /* In branch gnucash-1.6, must set currency after splits were inserted ... */
+      xaccTransSetCurrency (trans, currency);
+      xaccTransCommitEdit (trans);
+   }
+
+#if 0
+/* hack alert !! deal with kvp later -- huge sucking sound ! */
+   /* restore any kvp data associated with the transaction and splits */
+   for (node=xaction_list; node; node=node->next)
+   {
+      Transaction *trans = (Transaction *) node->data;
+      GList *engine_splits, *snode;
+
+      trans->kvp_data = pgendKVPFetch (be, &(trans->guid), trans->kvp_data);
+   
+      engine_splits = xaccTransGetSplitList(trans);
+      for (snode = engine_splits; snode; snode=snode->next)
       {
-         engine_data_is_newer = pgendCopyTransactionToEngine (be, trans_guid);
-         trans = xaccTransLookup (trans_guid);
-         trans->marker = 1;
-         PINFO ("copy result=%d", engine_data_is_newer);
-      }
-      else
-      {
-         PINFO ("avoided scan");
-         engine_data_is_newer = 1;
+         Split *s = snode->data;
+         s->kvp_data = pgendKVPFetch (be, &(s->guid), s->kvp_data);
       }
 
-      /* if we restored this transaction from the db, scan over the accounts 
-       * it affects and see how far back the data goes.
-       */
-      if (0 > engine_data_is_newer) 
+      xaccTransCommitEdit (trans);
+   }
+#endif
+
+   /* run the fill-out algorithm */
+   for (node=xaction_list; node; node=node->next)
+   {
+      Transaction *trans = (Transaction *) node->data;
+      GList *split_list, *snode;
+      Timespec ts;
+
+      ts = xaccTransRetDatePostedTS (trans);
+
+      /* Back off by a second to disambiguate time.
+       * This is safe, because the fill-out will recurse
+       * if something got into this one-second gap. */
+      ts.tv_sec --; 
+      split_list = xaccTransGetSplitList (trans);
+      for (snode=split_list; snode; snode=snode->next)
       {
-         GList *split_list, *snode;
-         Timespec ts;
+         int found = 0;
+         Split *s = (Split *) snode->data;
+         Account *acc = xaccSplitGetAccount (s);
 
-         ts = xaccTransRetDatePostedTS (trans);
-
-         /* Back off by a second to disambiguate time.
-          * This is safe, because the fill-out will recurse
-          * if something got into this one-second gap. */
-         ts.tv_sec --; 
-         split_list = xaccTransGetSplitList (trans);
-         for (snode=split_list; snode; snode=snode->next)
+         /* lets see if we have a record of this account already */
+         for (anode = acct_list; anode; anode = anode->next)
          {
-            int found = 0;
-            Split *s = (Split *) snode->data;
-            Account *acc = xaccSplitGetAccount (s);
-
-            /* lets see if we have a record of this account already */
-            for (anode = acct_list; anode; anode = anode->next)
+            AcctEarliest * ae = (AcctEarliest *) anode->data;
+            if (ae->acct == acc) 
             {
-               AcctEarliest * ae = (AcctEarliest *) anode->data;
-               if (ae->acct == acc) 
+               if (0 > timespec_cmp(&ts, &(ae->ts)))
                {
-                  if (0 > timespec_cmp(&ts, &(ae->ts)))
-                  {
-                     ae->ts = ts;
-                  }
-                  found = 1;
-                  break;
+                  ae->ts = ts;
                }
-            }
-
-            /* if not found, make note of this account, and the date */
-            if (0 == found)
-            {
-               AcctEarliest * ae = g_new (AcctEarliest, 1);
-               ae->acct = acc;
-               ae->ts = ts;
-               acct_list = g_list_prepend (acct_list, ae);
+               found = 1;
+               break;
             }
          }
+
+         /* if not found, make note of this account, and the date */
+         if (0 == found)
+         {
+            AcctEarliest * ae = g_new (AcctEarliest, 1);
+            ae->acct = acc;
+            ae->ts = ts;
+            acct_list = g_list_prepend (acct_list, ae);
+         }
       }
-      xaccGUIDFree (trans_guid);
    }
-   g_list_free(xaction_list);
+   g_list_free (xaction_list);
 
    if (NULL == acct_list) return;
 
@@ -422,15 +467,15 @@ pgendFillOutToCheckpoint (PGBackend *be, const char *query_string)
        * GetBalance goes to less-then-or-equal-to because of the BETWEEN
        * that appears in the gncSubTotalBalance sql function. */
       p = be->buff; *p = 0;
-      p = stpcpy (p, "SELECT DISTINCT gncEntry.transGuid from gncEntry, gncTransaction WHERE "
-                     "   gncEntry.transGuid = gncTransaction.transGuid AND accountGuid='");
+      p = stpcpy (p, "SELECT DISTINCT gncTransaction.* from gncEntry, gncTransaction WHERE "
+                     "   gncEntry.transGuid = gncTransaction.transGuid AND gncEntry.accountGuid='");
       p = guid_to_string_buff(xaccAccountGetGUID(ae->acct), p);
       p = stpcpy (p, "' AND gncTransaction.date_posted > '");
       p = gnc_timespec_to_iso8601_buff (ae->ts, p);
       p = stpcpy (p, "';");
      
       pgendFillOutToCheckpoint (be, be->buff);
-
+      
       g_free (ae);
    }
    g_list_free(acct_list);
@@ -460,8 +505,23 @@ pgendRunQuery (Backend *bend, Query *q)
 
    /* stage transactions, save some postgres overhead */
    xaccGroupBeginStagedTransactionTraversals (be->topgroup);
+
+   /* We will be doing a bulk insertion of transactions below.
+    * We can gain a tremendous performance improvement,
+    * for example, a factor of 10x when querying 3000 transactions,
+    * by opening all accounts for editing before we start, and
+    * closing them all only after we're done.  This is because
+    * an account must be open for editing in order to insert a split, 
+    * and when the commit is made, the splits are sorted in date order. 
+    * If we're sloppy, then there's an ordering for every insertion.
+    * By defering the Commit, we defer the sort, and thus save gobs.
+    * Of course, this hurts 'shallow' queries some, but I beleive 
+    * by not very much.
+    */
    ncalls = 0;
+   xaccAccountGroupBeginEdit(be->topgroup);
    pgendFillOutToCheckpoint (be, sql_query_string);
+   xaccAccountGroupCommitEdit(be->topgroup);
    PINFO ("number of calls to fill out=%d", ncalls);
 
    sql_Query_destroy(sq);
@@ -483,9 +543,25 @@ pgendRunQuery (Backend *bend, Query *q)
  *    CPU and memory-burner; its use is not suggested for anything
  *    but single-user mode.
  *
- *    To add injury to insult, this routine fetches in a rather 
- *    inefficient manner, in particular, the account query.
+ *    NB. This routine has been obsoleted by the more efficient
+ *    pgendGetMassTransactions().  We keep it around here ...
+ *    for a rainy day...
  */
+static gpointer
+get_all_trans_cb (PGBackend *be, PGresult *result, int j, gpointer data)
+{
+   GList *node, *xaction_list = (GList *) data;
+   GUID *trans_guid;
+
+   /* find the transaction this goes into */
+   trans_guid = xaccGUIDMalloc();
+   *trans_guid = nullguid;  /* just in case the read fails ... */
+   string_to_guid (DB_GET_VAL("transGUID",j), trans_guid);
+
+   xaction_list = g_list_prepend (xaction_list, trans_guid);
+   return xaction_list;
+}
+
 
 static void
 pgendGetAllTransactions (PGBackend *be, AccountGroup *grp)
@@ -496,7 +572,7 @@ pgendGetAllTransactions (PGBackend *be, AccountGroup *grp)
    pgendDisable(be);
 
    SEND_QUERY (be, "SELECT transGuid FROM gncTransaction;", );
-   xaction_list = pgendGetResults (be, query_cb, xaction_list);
+   xaction_list = pgendGetResults (be, get_all_trans_cb, xaction_list);
 
    /* restore the transactions */
    for (node=xaction_list; node; node=node->next)
@@ -567,10 +643,12 @@ pgendSync (Backend *bend, AccountGroup *grp)
 
    be->version_check = (guint32) time(0);
 
-   /* for the multi-user modes, we allow a save only once,
-    * when the database is created for the first time */
+   /* For the multi-user modes, we allow a save only once,
+    * when the database is created for the first time.
+    * Ditto for the single-user update mode: it should never
+    * wander out of sync.
+    */
    if ((MODE_SINGLE_FILE != be->session_mode) &&
-       (MODE_SINGLE_UPDATE != be->session_mode) &&
        (FALSE == be->freshly_created_db))
    {
       LEAVE("no sync");
@@ -597,7 +675,7 @@ pgendSync (Backend *bend, AccountGroup *grp)
    else
    {
       /* in single user mode, read all the transactions */
-      pgendGetAllTransactions (be, grp);
+      pgendGetMassTransactions (be, grp);
    }
 
    /* re-enable events */
@@ -990,7 +1068,9 @@ pgendSessionEnd (PGBackend *be)
 
    if (!be->sessionGuid) return;
 
+   /* vacuuming w/ analyze can improve performance 20% */
    p = be->buff; *p=0;
+   p = stpcpy (p, "VACUUM ANALYZE;\n");
    p = stpcpy (p, "UPDATE gncSession SET time_off='NOW' "
                   "WHERE sessionGuid='");
    p = stpcpy (p, be->session_guid_str);
@@ -1029,7 +1109,8 @@ pgend_session_end (Backend *bend)
           * it might be opened in multi-user mode next time. Thus, update
           * the account balance checkpoints just in case. 
           */
-         pgendGroupRecomputeAllCheckpoints (be, be->topgroup);
+         /* hack alert -- disable for now, performance drain */
+         /* pgendGroupRecomputeAllCheckpoints (be, be->topgroup); */
          break;
 
       case MODE_POLL:
@@ -1136,7 +1217,7 @@ pgend_price_load_poll (Backend *bend)
 /* The pgend_book_load_single() routine loads the engine with
  *    data from the database.  Used only in single-user mode,
  *    it loads account *and* transaction data.  Single-user
- *    mode doesn't require balance checkpoingts, to these are
+ *    mode doesn't require balance checkpoints, to these are
  *    not handled.
  */
 
@@ -1154,7 +1235,7 @@ pgend_book_load_single (Backend *bend)
 
    pgendKVPInit(be);
    grp = pgendGetAllAccounts (be, NULL);
-   pgendGetAllTransactions (be, grp);
+   pgendGetMassTransactions (be, grp);
 
    /* re-enable events */
    pgendEnable(be);
