@@ -3,11 +3,23 @@
 # example script showing how to use the Quote perl module.
 # gets prices for some stocks, for some mutual funds
 
-use Quote;
+# Note that this example uses the meta-level "fetch" command.  We do
+# NOT used that in Gnucash because it's behavior is unpredictable If
+# the given method/exchange doesn't work, it'll fall back to other
+# methods, and I've seen no guarantee that all exchanges treat all
+# symbols the same.  So in Gnucash, we use the backend methods
+# directly, i.e. $quoter->fidelity_direct("IBM", "LNUX");, etc.  The
+# documentation page for each Finance::Quote sub-module describes how
+# to call it directly without fallbacks.
+
+use Finance::Quote;
+
+# Create a quote object.
+my $quoter = Finance::Quote->new();
 
 # -----------------------------------
 # get quotes for two stocks ...
-%quotes = Quote::yahoo ("IBM", "SGI"); 
+%quotes = $quoter->fetch("yahoo","IBM", "SGI"); 
 
 # print some selected values 
 print "NYSE by Yahoo: ", $quotes {"IBM", "name"},  
@@ -28,7 +40,7 @@ print "\n\n";
 # -----------------------------------
 # get quotes from Fidelity Investments
 @funds = ("FGRIX", "FNMIX", "FASGX", "FCONX");
-%quotes = Quote::fidelity (@funds);
+%quotes = $quoter->fetch("fidelity",@funds);
 
 foreach $f (@funds) {
      $name = $quotes {$f, "name"};
@@ -39,7 +51,7 @@ print "\n\n";
 
 # -----------------------------------
 @funds = ("FGRXX");
-%quotes = Quote::fidelity (@funds);
+%quotes = $quoter->fetch("fidelity",@funds);
 
 print "Not all funds have a NAV; some have Yeilds:\n";
 foreach $f (@funds) {
@@ -52,7 +64,7 @@ print "\n\n";
 # -----------------------------------
 # demo T. Rowe Price -- same as above
 @funds = ("PRFDX", "PRIDX");
-%quotes = Quote::troweprice (@funds);
+%quotes = $quoter->fetch("troweprice",@funds);
 
 foreach $f (@funds) {
      $nav = $quotes {$f, "nav"};
@@ -66,7 +78,7 @@ print "\n\n";
 
 # demo for ASX.  Grab the price of Coles-Myer and Telstra
 @funds = ("CML","TLS");
-%quotes = Quote::asx(@funds);
+%quotes = $quoter->fetch("australia",@funds);
 foreach $f (@funds) {
 	print "ASX Price of $f is ".$quotes{$f,"last"}." at ".
 	      $quotes{$f,"date"}."\n";
