@@ -24,10 +24,9 @@
 #include <gnome.h>
 
 #include "splitreg.h"
-#include "table-gnome.h"
 #include "table-allgui.h"
 
-#define CELL_VPADDING 5
+#define CELL_VPADDING 3
 #define CELL_HPADDING 4
 
 
@@ -43,101 +42,30 @@
 #define GNUCASH_IS_SHEET(o)    (GTK_CHECK_TYPE((o), GNUCASH_TYPE_SHEET))
 
 
-enum {
-        GNUCASH_CURSOR_HEADER = 0,
-        GNUCASH_CURSOR_SINGLE,
-        GNUCASH_CURSOR_DOUBLE,
-        GNUCASH_CURSOR_TRANS,
-        GNUCASH_CURSOR_SPLIT,
-        GNUCASH_CURSOR_LAST,
-};
-
-
-typedef enum {
-        GNUCASH_ALIGN_TOP,
-        GNUCASH_ALIGN_BOTTOM,
-        GNUCASH_ALIGN_SAME,   /* keep the alignment the same */
-} GnucashSheetAlignment;
-
-
-typedef struct _CellLayoutInfo CellLayoutInfo;
-
-typedef struct 
-{
-        /* totals, in pixels */
-        gint height;
-        gint width;
-
-        /* per cell parameters */
-        
-        gint **pixel_heights;    /* in pixels, may be zero if
-                                    row/column not displayed */
-        gint **pixel_widths;
-
-        gint **origin_x;   /* the origin of the cell */
-        gint **origin_y;
-
-        gint nrows, ncols;
-        gint refcount;
-        
-} CellDimensions;
-
-typedef struct
-{
-        gint nrows;
-        gint ncols;
-
-        gint reg_type;
-        gint cursor_type;
-
-        /* this one comes from the cellblock */
-        gint **widths;              /* in characters */
-
-        CellLayoutInfo *layout_info;
-        CellDimensions *dimensions;
-
-        gchar ***labels;              /* for the header */
-        GdkFont *header_font;          
-        
-        GtkJustification **alignments;
-
-        /* per cell fonts;  if NULL, use the grid normal font */
-        GdkFont ***fonts;
-
-        GdkColor ***active_bg_color;
-        GdkColor ***inactive_bg_color;
-        int **borders;
-
-        gint refcount;
-} SheetBlockStyle;
-
+typedef struct _SheetBlockStyle SheetBlockStyle;
 
 typedef struct  
 {
-        /* the virtual row/column in the table this block
-           is associated to */
-        gint virt_row; 
-        gint virt_col;
-
-        /*  The style for this block, derived from the handlers for
-            the virt row/col */
+        /* The style for this block */
         SheetBlockStyle *style;
 
-        GdkColor ***fg_colors;
-        GdkColor ***bg_colors;                
+        gint origin_x; /* x origin of block */
+        gint origin_y; /* y origin of block */
 
-        /*  the text of the block;  a num_phys_row by num_phys_cols array */
-        gchar ***entries;
+        gboolean visible; /* is block visible */
 } SheetBlock;
 
 
-typedef struct {
+typedef struct
+{
         GnomeCanvas canvas;
 
         GtkWidget *window;
 
+        GtkWidget *popup;
+        gpointer popup_data;
+
         Table *table;
-        SplitRegister *split_register;
 
         GtkWidget *reg;
 
@@ -148,26 +76,26 @@ typedef struct {
         GnomeCanvasItem *cursor;
         GnomeCanvasItem *grid;
 
-        SheetBlockStyle *cursor_style[GNUCASH_CURSOR_LAST];
+        CellBlock *cursors[NUM_CURSOR_TYPES];
+        SheetBlockStyle *cursor_styles[NUM_CURSOR_TYPES];
 
         /* some style information associated to a sheet */
-        GHashTable *layout_info_hash_table;
         GHashTable *dimensions_hash_table;
 
-        GHashTable *blocks;
+        GTable *blocks;
 
         GnomeCanvasItem *item_editor;
         GtkWidget *entry;   
 
+        gboolean input_cancelled;
+
         gint top_block;  /* maybe not fully visible */
         gint bottom_block;
-        gint left_block, right_block;
+        gint left_block;
+        gint right_block;
 
-        gint top_block_offset; 
-        gint left_block_offset;
-
-        gint default_width;
-        gint default_height;
+        gint num_visible_blocks;
+        gint num_visible_phys_rows;
 
         gint width;  /* the width in pixels of the sheet */
         gint height;
@@ -175,7 +103,7 @@ typedef struct {
         gint window_height;
         gint window_width;
 
-        gint alignment;
+        gint cell_borders;
 
         gint editing;
 
@@ -186,12 +114,11 @@ typedef struct {
         guint delete_signal;
         guint changed_signal;
 
-        gint smooth_scroll;
         GtkAdjustment *hadj, *vadj;
 } GnucashSheet;
 
 
-typedef struct 
+typedef struct
 {
         GtkTable table;
 
@@ -203,11 +130,13 @@ typedef struct
 
 
 GtkType    gnucash_sheet_get_type (void);
-GtkWidget *gnucash_sheet_new 	 (Table *table);
+GtkWidget *gnucash_sheet_new (Table *table);
 
-void gnucash_sheet_table_load (GnucashSheet *sheet);
+void gnucash_sheet_table_load (GnucashSheet *sheet, gboolean do_scroll);
 
-GtkType    gnucash_register_get_type (void);
+void gnucash_sheet_recompute_block_offsets (GnucashSheet *sheet);
+
+GtkType gnucash_register_get_type (void);
 
 /* this already has scrollbars attached */
 GtkWidget *gnucash_register_new (Table *table);
@@ -216,55 +145,53 @@ void gnucash_sheet_set_top_block (GnucashSheet *sheet, int new_top_block,
                                   gint align);
 
 
-SheetBlock *gnucash_sheet_get_block (GnucashSheet *sheet, gint vrow,
-				     gint vcol);
-gint
-gnucash_sheet_col_max_width (GnucashSheet *sheet, gint virt_col, gint cell_col);
+SheetBlock *gnucash_sheet_get_block (GnucashSheet *sheet,
+                                     VirtualCellLocation vcell_loc);
 
-gint gnucash_sheet_col_get_distance(GnucashSheet *sheet, int v_row, int col_a, int col_b);
-
-gint gnucash_sheet_row_get_distance (GnucashSheet *sheet, int row_a,
-				     int row_b);
+gint gnucash_sheet_col_max_width (GnucashSheet *sheet,
+                                  gint virt_col, gint cell_col);
 
 void gnucash_sheet_redraw_all (GnucashSheet *sheet);
-void gnucash_sheet_redraw_block (GnucashSheet *sheet, gint row, gint col);
 
-void gnucash_sheet_cursor_set (GnucashSheet *gsheet,
-                               int virt_row, int virt_col,
-			       int cell_row, int cell_col);
+void gnucash_sheet_redraw_block (GnucashSheet *sheet,
+                                 VirtualCellLocation vcell_loc);
+
+void gnucash_sheet_cursor_set (GnucashSheet *gsheet, VirtualLocation virt_loc);
 
 const char * gnucash_sheet_modify_current_cell(GnucashSheet *sheet,
 					       const gchar *new_text);
 
-void gnucash_sheet_block_set_from_table (GnucashSheet *sheet, gint virt_row,
-                                         gint virt_col);
+gboolean gnucash_sheet_block_set_from_table (GnucashSheet *sheet,
+                                             VirtualCellLocation vcell_loc);
+
 void gnucash_sheet_set_scroll_region (GnucashSheet *sheet);
 
 void gnucash_sheet_cursor_set_from_table (GnucashSheet *sheet,
-                                          gncBoolean do_scroll);
-
-void gnucash_sheet_move_cursor (GnucashSheet *sheet, int col, int row);
-
-void gnucash_sheet_set_cursor_bounds (GnucashSheet *sheet,
-				      int start_col, int start_row,
-				      int end_col,   int end_row);
+                                          gboolean do_scroll);
 
 void gnucash_sheet_compute_visible_range (GnucashSheet *sheet);
 
-void gnucash_sheet_block_pixel_origin (GnucashSheet *sheet,
-                                       gint vrow, gint vcol,
-                                       gint *x, gint *y);
-
 void gnucash_sheet_make_cell_visible (GnucashSheet *sheet,
-				      gint virt_row, gint virt_col,
-				      gint cell_row, gint cell_col);
+                                      VirtualLocation virt_loc);
+
+void gnucash_sheet_show_range (GnucashSheet *sheet,
+                               VirtualCellLocation start_loc,
+                               VirtualCellLocation end_loc);
+
+void gnucash_sheet_set_cursor (GnucashSheet *sheet, CellBlock *cursor);
+
 void gnucash_sheet_update_adjustments (GnucashSheet *sheet);
 
-void gnucash_register_goto_virt_row_col (GnucashRegister *reg,
-                                         int v_row, int v_col);
+void gnucash_register_goto_virt_cell (GnucashRegister *reg,
+                                      VirtualCellLocation vcell_loc);
+
+void gnucash_register_goto_virt_loc (GnucashRegister *reg,
+                                     VirtualLocation virt_loc);
 
 void gnucash_register_goto_next_virt_row (GnucashRegister *reg);
 
+void gnucash_register_goto_next_trans_row (GnucashRegister *reg);
+
 void gnucash_register_attach_popup(GnucashRegister *reg, GtkWidget *popup,
                                    gpointer data);
 
@@ -275,28 +202,18 @@ void gnucash_register_copy_clipboard (GnucashRegister *reg);
 void gnucash_register_paste_clipboard (GnucashRegister *reg);
 
 
-
-typedef struct {
+typedef struct
+{
         GnomeCanvasClass parent_class;
-
-        gint (*traverse)       	(GnucashSheet *sheet,
-                                 gint row, gint column,
-                                 gint *new_row, gint *new_column);
-        
-        gint (*deactivate)	(GnucashSheet *sheet,
-				 gint row, gint column);
-        
-        gint (*activate) 	(GnucashSheet *sheet,
-				 gint row, gint column);
-        
 } GnucashSheetClass;
 
 
-typedef struct {
+typedef struct
+{
         GtkTableClass parent_class;
 
         void (*activate_cursor) (GnucashRegister *reg);
-
+        void (*redraw_all)      (GnucashRegister *reg);
 } GnucashRegisterClass;
 
 #endif
