@@ -17,11 +17,7 @@
 
 (require 'hash-table)
 
-
-;; Provides pointer-token-null? if needed (g-wrap >= 0.9.4 doesn't provide)
-(if (not (defined? 'pointer-token-null?))
-   (define (pointer-token-null? ptr)
-     (eq? ptr #f)))
+(use-modules (ice-9 syncase))
 
 (define (gnc:error->string tag args)
   (define (write-error port)
@@ -36,64 +32,34 @@
                 (display "\n\n" port)))))
     (false-if-exception
      (apply display-error (fluid-ref the-last-stack) port args))
-    (display-backtrace (fluid-ref the-last-stack) port))
-
+    (display-backtrace (fluid-ref the-last-stack) port)
+    (force-output port))
+  
   (false-if-exception
    (call-with-output-string write-error)))
 
 
-(define gnc:register-translatable-strings #f)
-(define gnc:save-translatable-strings #f)
-
-(let ((string-hash (make-hash-table 313)))
-
-  (define (register . strings)
-    (if (gnc:debugging?)
-        (for-each
-         (lambda (string)
-           (if (and (string? string) (> (string-length string) 0))
-               (hash-set! string-hash string #t)))
-         strings)))
-
-  (define (save file)
-    (let ((port (open file (logior O_WRONLY O_CREAT O_TRUNC))))
-      (if port
-          (begin
-            (hash-for-each
-             (lambda (string not-used)
-               (display "_(" port)
-               (write string port)
-               (display ")\n" port))
-             string-hash)
-            (close port)))))
-
-  (set! gnc:register-translatable-strings register)
-  (set! gnc:save-translatable-strings save))
-
-(if (gnc:debugging?)
-    (define (gnc:gettext string)
-      (gnc:register-translatable-strings string)
-      (gnc:gettext-helper string))
-    (define gnc:gettext gnc:gettext-helper))
-
+;; gettext functions
+(define gnc:gettext gnc:gettext-helper)
 (define gnc:_ gnc:gettext)
+(define _ gnc:gettext)
+(define-syntax N_
+  (syntax-rules ()
+    ((_ x) x)))
 
 
 ;; This database can be used to store and retrieve translatable
-;; strings. If debugging is true, the stored strings are registered so
-;; they can be saved to a string database. Strings that are returned
-;; by the lookup function are translated with gettext.
+;; strings. Strings that are returned by the lookup function are
+;; translated with gettext.
 (define (gnc:make-string-database)
 
   (define string-hash (make-hash-table 23))
 
   (define (lookup key)
-    (hash-ref string-hash key))
+    (_ (hash-ref string-hash key)))
 
   (define (store key string)
-    (if (gnc:debugging?)
-        (gnc:register-translatable-strings string))
-    (hash-set! string-hash key (gnc:_ string)))
+    (hash-set! string-hash key string))
 
   (define (dispatch message . args)
     (let ((func (case message
