@@ -3,7 +3,7 @@
 ;;;  print a check from a transaction. 
 ;;;
 ;;;  Copyright 2000 Bill Gribble <grib@billgribble.com>
-;;;  $Id: print-check.scm,v 1.4 2000/03/31 09:35:46 peticolas Exp $
+;;;  $Id: print-check.scm,v 1.6 2001/01/17 23:21:36 peticolas Exp $
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (gnc:support "printing/print-check.scm")
@@ -23,32 +23,32 @@
      date-format
      custom-info)))
 
-(define (print-check-format? self)
-  (eq? (simple-obj-type self) 'print-check-format))
+(define print-check-format?
+  (record-predicate <print-check-format>))
 
-(define (print-check-format:format self)
-  (simple-obj-getter self <print-check-format> 'format))
+(define print-check-format:format
+  (simple-obj-getter <print-check-format> 'format))
 
-(define (print-check-format:set-format! self value)
-  (simple-obj-setter self <print-check-format> 'format value))
+(define print-check-format:set-format!
+  (simple-obj-setter <print-check-format> 'format))
 
-(define (print-check-format:position self)
-  (simple-obj-getter self <print-check-format> 'position))
+(define print-check-format:position
+  (simple-obj-getter <print-check-format> 'position))
 
-(define (print-check-format:set-position! self value)
-  (simple-obj-setter self <print-check-format> 'position value))
+(define print-check-format:set-position!
+  (simple-obj-setter <print-check-format> 'position))
 
-(define (print-check-format:date-format self)
-  (simple-obj-getter self <print-check-format> 'date-format))
+(define print-check-format:date-format
+  (simple-obj-getter <print-check-format> 'date-format))
 
-(define (print-check-format:set-date-format! self value)
-  (simple-obj-setter self <print-check-format> 'date-format value))
+(define print-check-format:set-date-format!
+  (simple-obj-setter <print-check-format> 'date-format))
 
-(define (print-check-format:custom-info self)
-  (simple-obj-getter self <print-check-format> 'custom-info))
+(define print-check-format:custom-info
+  (simple-obj-getter <print-check-format> 'custom-info))
 
-(define (print-check-format:set-custom-info! self value)
-  (simple-obj-setter self <print-check-format> 'custom-info value))
+(define print-check-format:set-custom-info!
+  (simple-obj-setter <print-check-format> 'custom-info))
    
 (define (make-print-check-format fmt pos dateformat cust)
   (let ((retval (make-simple-obj <print-check-format>)))
@@ -136,7 +136,7 @@
         (gnc:print-session-text ps (number-to-words amount 100)))
       
       (gnc:print-session-done ps)       
-      (gnc:print-dialog-create ps)))
+      (gnc:print-session-print ps)))
 
   (gnc:print-check-dialog-create print-check-callback))
 
