@@ -1,5 +1,5 @@
 ;; -*-scheme-*-
-;; $Id: txf-export-help.scm,v 1.1.2.2 2001/01/05 02:02:06 peticolas Exp $
+;; $Id: txf-export-help.scm,v 1.4 2001/04/30 01:58:24 peticolas Exp $
 ;;
 ;;  Richard -Gilligan- Uschold
 ;; These are help strings for each TXF code. See taxtxf.scm and
@@ -45,18 +45,15 @@
     (N273 . "Prescription medicines, eyeglasses, contact lenses, hearing aids. Over-the-counter medicines are not deductable.")
     (N274 . "Lodging expenses while away from home to receive medical care in a hospital or a medical care facility related to a hospital. Do not include more than $50 a night for each eligible person. Ambulance service and other travel costs to get medical care.")
     (N275 . "State income taxes paid this year for a prior year. Includ any part of a prior year refund that you chose to have credited to this years state income taxes.")
-    (N544 . "Local income taxes that were not withheld from your salary, such as local income taxes you paid this year for a prior year.
-")
+    (N544 . "Local income taxes that were not withheld from your salary, such as local income taxes you paid this year for a prior year.")
     (N276 . "Include taxes (state, local, or foreign) you paid on real estate you own that was not used for business, but only if the taxes are based on the assessed value of the property. Do not include taxes charged for improvements that tend to increase the value of your property (for example, an assessment to build a new sidewalk).")
     (N535 . "Enter personal property tax you paid, but only if it is based on value alone. Example: You paid a fee for the registration of your car. Part of the fee was based on the car s value and part was based on its weight. You may deduct only the part of the fee that is based on the car s value.")
     (N277 . "Other taxes paid not included under state and local income taxes, real estate taxes, or personal property taxes. You may want to take a credit for the foreign tax instead of a deduction.")
     (N280 . "Contributions or gifts by cash or check you gave to organizations that are religious, charitable, educational, scientific, or literary in purpose. You may also deduct what you gave to organizations that work to prevent cruelty to children or animals. For donations of $250 or more, you must have a statement from the charitable organization showing the amount donated and the value of goods or services you received.")
-    (N485 . "
-The fair market value of donated property, such as used clothing or furniture.")
+    (N485 . "The fair market value of donated property, such as used clothing or furniture.")
     (N281 . "Fees you paid for preparation of your tax return, including fees paid for filing your return electronically.")
-    (N282 . "Investment interest is interest paid on money you borrowed that is allocable to property held for investment. It does not include any interest allocable to passive activities or to securities that generate tax-exempt income.
-")
-82    (N283 . "Home mortgage interest and points reported to you on Form 1098. The interest could be on a first or second mortgage, home equity loan, or refinanced mortgage.")
+    (N282 . "Investment interest is interest paid on money you borrowed that is allocable to property held for investment. It does not include any interest allocable to passive activities or to securities that generate tax-exempt income.")
+    (N283 . "Home mortgage interest and points reported to you on Form 1098. The interest could be on a first or second mortgage, home equity loan, or refinanced mortgage.")
     (N284 . "Generally, you must deduct points you paid to refinance a mortgage over the life of the loan. If you used part of the proceeds to improve your main home, you may be able to deduct the part of the points related to the improvement in the year paid. See Pub. 936 Use this line for points not reported on Form 1098.")
     (N484 . "Insurance premiums for medical and dental care, medical doctors, dentists, eye doctors, surgeons, X-ray, laboratory services, hospital care, etc. See IRS Pub 502.")
     (N486 . "Safety equipment, small tools, and supplies you needed for your job; Uniforms required by your employer and which you may not usually wear away from work; subscriptions to professional journals; job search expenses; certain educational expenses. You may need to file Form 2106.")
